#! /bin/sh
#
# patchymf.sh release 1.30
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/
#
# This script modifies BeOS R4.5.x YMF724 driver
# for use with some other Yamaha PCI audio chipsets
# and installs it to appropriate place.
#
# 1.00 first release
# 1.10 R4.5 experimental driver supported
# 1.20 R4.5.2 driver supported
# 1.30 YMF754 supported (untested)
#
# Get YMF724F specifications at:
# http://www.yamahayst.com/pcaudio/products.htm
#
# PCI vendor ID: 1073h (Yamaha)
# PCI device IDs:
# YMF724D (DS-1)  0004h
# YMF724F (DS-1)  000Dh
# YMF740C (DS-1L) 000Ch
# YMF744B (DS-1S) 0010h
# YMF754  (DS-1E) 0012h

TMP=/tmp
DIR=/system/add-ons/kernel/drivers/bin
FILE=ymf724
# R4.5, R4.5+update1, R4.5+update2
OLDSUM45=602392030
# R4.5.2 CD
OLDSUM452=1663685498
# R4.5(.2) experimental driver
OLDSUM45E=83570629

# check parameters

if [ $# -ne 1 ]
then
	echo "Usage: $0 {ymf724f|ymf740|ymf744|ymf754}"
	exit 1
fi

case $1 in
ymf724f) ID=\\015;;
ymf740)  ID=\\014;;
ymf744)  ID=\\020;;
ymf754)  ID=\\022;;
*)
	echo "Unknown chipset: $1"
	exit 1
esac

# check original driver

if [ ! -f $DIR/$FILE ]
then
	echo "Original driver $DIR/$FILE not present."
	exit 1
fi

# if you've got different version of ymf724 driver,
# change OFS1/OFS2 values so that they point to device ID in driver
# (73 10 04 00)

OLDSUM=`cksum $DIR/$FILE | cut -d ' ' -f 1`
case "$OLDSUM" in
$OLDSUM45)
	echo "R4.5 standard YMF724 driver detected."
	OFS1=4634
	OFS2=4922
	;;
$OLDSUM452)
	echo "R4.5.2 standard YMF724 driver detected."
	OFS1=4634
	OFS2=4922
	;;
$OLDSUM45E)
	echo "R4.5(.2) experimental YMF724 driver (w/ MIDI and game port support) detected."
	OFS1=6181
	OFS2=6517
	;;
*)
	echo "Unsupported version of YMF724 driver."
	exit 1
esac

# apply patch to driver

cp $DIR/$FILE $TMP/$1
chmod u+w $TMP/$1
/bin/echo -n $ID | dd of=$TMP/$1 bs=1 seek=$OFS1 conv=notrunc
/bin/echo -n $ID | dd of=$TMP/$1 bs=1 seek=$OFS2 conv=notrunc

# checking new driver is omitted because original driver is retained as-is
# and sound driver is not fatal to system

# install modified driver

cd ~/config/add-ons/kernel/drivers
mv $TMP/$1 bin/$1
# refrain mimeset and you can easily compare new driver with original
# mimeset -f bin/$1

mkdir -p dev/audio/old
[ -h dev/audio/old/$1 ] || ln -s ../../../bin/$1 dev/audio/old/$1

echo "All done."
